'use strict'
############################################################
#
# 상자수염그림
#
do ()->
  d3.svg.box = ()->
    scale = (d)-> d
    value = (d)-> d
    values = (d)-> d
    width = 30
    whiskerWidth = 30
    withHist = false

    # 최대・최소인 수염을 그린다
    whisker = (source, target)->
      ()->
        s = this.append('g').classed('whisker', true)
        s.append('path').attr(
          d: "M#{source[0]},#{source[1]}L#{target[0]},#{target[1]}"
        )
        s.append('path').attr(
          d: "M0,#{target[1]}L#{target[0] * 2},#{target[1]}"
        )

    # 히스토 그램을 추가하는 함수 
    hist = (data, height)->
      ()->
        histogram = d3.layout.histogram().bins(20)
        histData = histogram(data)

        y = d3.scale.linear().domain(d3.extent(histData, (d)-> d.y)).range([0, width / 2])
        height = scale(0) - scale(histData[0].dx)

        this.selectAll('rect').data(histogram(data)).enter().append('rect')
          .attr('x', 0).attr('y', (d)-> scale(d.x) - height)
          .attr('width', (d)-> y(d.y)).attr('height', height)
          .style('opacity', 0.5).attr('stroke', 'none').attr('fill', 'grey')

    # 다섯 개의 값(중앙값, 최댓값, 최솟값, 1사분위, 3사분위)을 계산한다
    getFive = (data)->
      max: Math.max.apply Math, data
      min: Math.min.apply Math, data
      first: d3.quantile data, 0.25
      third: d3.quantile data, 0.75
      median: d3.median data

    # 스케일을 적용한 닷서 개의 값을 계산한다
    getScaledFive = (five, scale)->
      iqr = five.third - five.first
      min = scale(five.min)
      mim = scale(five.first - 1.5 * iqr) if five.min < five.first - 1.5 * iqr
      max = scale(five.max)
      max = scale(five.third + 1.5 * iqr) if five.max > five.third + 1.5 * iqr

      min: min
      max: max
      median: scale(five.median)
      first: scale(five.first)
      third: scale(five.third)

    # box본체 
    box = (g)->
      # 기본적인 배색
      # 데이터 취득
      g.each ()->
        g = d3.select(this)

        data = values(this.__data__)

        # x축, y축의 값을 가져온다
        arr = data.map(value).filter((d)-> not isNaN d).sort((a, b)-> a - b)

        # 다섯 개의 값을 계산
        five = getFive(arr)

        # Interquartile Range를 계산 
        iqr = five.third - five.first

        # 특잇값을 가져온다
        outlier = data.filter((d)->
          not ((five.first - 1.5 * iqr) < value(d) < (five.third + 1.5 * iqr)))

        # 스케일된 다섯 개의 값을 계산 
        scaled = getScaledFive(five, scale)

        # 높이 계산
        height = Math.abs scaled.third - scaled.first

        # rect 요소 그리기
        rect = g.append('rect').attr
          width: width, height: height, y: scaled.third, fill: 'none'

        # 중앙값 그리기
        medianLine = g.append('line').classed('median', true).attr(
          x1: 0, x2: width
          y1: scaled.median
          y2: scaled.median)

        # 최댓값・최솟값의 수염을 그린다
        g.call(whisker([width / 2, scaled.first],[width / 2, scaled.min]))
        g.call(whisker([width / 2, scaled.third],[width / 2, scaled.max]))

        # 특잇값 그리기
        g.selectAll('circle').data(outlier).enter().append('circle').attr(
          cx: width / 2, cy: ((d)-> scale(value(d))), r: 3, fill: 'none'
        )

        # 히스토그램 그리기
        if withHist
          g.append('g').call(hist(arr, height)).attr('transform', "translate(#{width / 2},0)")

    # 메소드 설정 
    box.scale = (_scale)->
      return scale unless _scale
      scale = _scale
      box
    box.value = (_value)->
      return value unless _value
      value = _value
      box
    box.values = (_values)->
      return values unless _values
      values = _values
      box
    box.width = (_width)->
      return width unless _width
      width = _width
      box
    box.whiskerWidth = (_width)->
      return whiskerWidth unless _width
      whiskerWidth = _width
      box
    box.withHist = (_bool)->
      return withHist unless _bool
      withHist = _bool
      box
    box
