'use strict'
# 그리기 영역 정보 
margin = new Margin(50)
[width, height] = [800, 500]

#------------------------------------------------------------
#
# ## getOrientation
#
# 방향을 계산하는 함수 
#
# * @param  d      오브젝트 링크오브젝트     오브젝트 source,target을 가진 링크 오브젝트 
# * @return object x, y 방향 벡터, 부호, 각도 오브젝트
#
getOrientation = (d)->
  dist =
    x: d.target.x - d.source.x
    y: d.target.y - d.source.y
  sign = if dist.x > 0 then 1 else -1
  angle = Math.atan(dist.y / dist.x) / Math.PI * 180 + (1 - sign) * 90 or 0

  x: dist.x
  y: dist.y
  sign: sign
  angle: angle

#------------------------------------------------------------
#
# ## drawStateTransition
#
# 상태전이도를 작성한다
#
# * @param  target        문자열 svg를 추가하는 요소의 XPath
# * @param  data          배열  데이터 매트릭스
#
drawStateTransition = (target, data)->
  # 라벨 배열 
  label = data.map (d)-> d.key
  # 링크 배열 
  links = []
  data.forEach (from, fromid)->
    from.values.forEach (to)->
      if to.value > 0
        links.push
          source: fromid
          target: label.indexOf(to.key)
          value:  to.value
  # force 레이아웃 생성 
  force = d3.layout.force()
    .size([width, height])
    .nodes(data).charge(- height)
    .links(links)
    .linkDistance((d)-> # 링크 거리를 확률에 대응한다 
      (1 - d.value / 100) * height / 2)

  # 색 추출 
  color = d3.scale.category10()
  # svg 추가 
  svg = d3.select(target).append('svg').attr(
    width: width + margin.width
    height: height + margin.height
  )
  # g 추가 
  main = svg.append('g').attr(
    width: width
    height: height
    transform: "translate(#{margin.left},#{margin.top})"
  )
  # 정의(ID=end-arrow)
  defs = svg.append('svg:defs')
  defs.append('marker').attr(
    id: 'end-arrow'
    viewBox: '0 0 10 10'
    refX: 6, refY: 4
    markerWidth: 6, markerHeight: 6
    orient: 'auto')
  .append('path').attr(
    d: 'M0,0L10,5L0,10z' # 삼각형 
    fill: 'grey'
    class: 'end-arrow')

  # 링크의 데이터 경로
  diagonal = d3.svg.diagonal()
  # 확률과 함께 표시할 화살표 경로 데이터 
  selfLink = ()-> # 자신에게서 자신으로 전이
    d3.svg.arc().innerRadius(20).outerRadius(25)(startAngle:Math.PI + 0.1, endAngle: 3 * Math.PI - 0.1)
  arrow = ()-> "M0,5L20,5L20,15L30,0L20,-15L20,-5L0,-5L0,5"

  # 링크 그리기 
  link = main.selectAll('g.link').data(links).enter().append('g').attr(
    class: 'link'
    opacity: .6
  )
  link.append('path').attr(
    class: 'line'
    fill: 'none'
    stroke: 'grey'
    'stroke-width': (d)-> # 확률에 따른 링크의 굵기 
      Math.sqrt d.value
  ).style('marker-end', 'url(#end-arrow)')
  link.append('text').text((d)-> "#{d.value}%")
  link.append('path').attr(
    class: 'arrow'
    d: arrow
    opacity: .3
  )

  # 노드 그리기 
  node = main.selectAll('g.node').data(data).enter().append('g').attr(
    attr: 'node'
    transform: (d)-> "translate(#{d.x},#{d.y})"
  )
  node.append('ellipse').attr(
    rx: 40, ry: 20
    fill: (d)-> color(d.key)
    opacity: .6
    cursor: 'pointer'
  )
  node.append('text').text((d)-> d.key).style('text-anchor', 'middle')
  node.call(force.drag) # 노드를 드래그할 수 있게 한다 

  # tick이벤트로 매번 노드와 링크를 수정한다 
  force.on 'tick', (d)->
    # 노드의 위치를 갱신
    node.attr(
      transform: (dd)-> "translate(#{dd.x},#{dd.y})"
    )
    # 링크ㅡ의 line 위치를 갱신 
    link.selectAll('path.line').attr(
      d: (dd)-> diagonal dd
    )
    # 링크의 확률전이 화살표의 위치 갱신 
    link.selectAll('path.arrow').attr(
      d: (dd)-> if dd.source.x is dd.target.x then selfLink(dd.source) else arrow()
      transform: (d)->
        orient = getOrientation(d)
        x = (d.source.x + d.target.x) / 2
        y = (d.source.y + d.target.y) / 2
        "translate(#{x},#{y + orient.sign * 20}) rotate(#{orient.angle})"
    )
    # 링크의 전이확률의 text 위치 갱신 
    link.selectAll('text').attr(
      dx: (d)->
        orient = getOrientation(d)
        (d.source.x + d.target.x) / 2
      dy: (d)->
        orient = getOrientation(d)
        (d.source.y + d.target.y) / 2 + orient.sign * 10
    ).style(
      'text-anchor': (d)-> if d.target.y - d.source.y > 0 then 'start' else 'end'
    )

  # force 계산 시작 
  force.start()

#------------------------------------------------------------
#
# 데이터를 읽어와서 그린다
#
d3.csv "./transition_matrix.csv",
  # 레코드 해석 함수 
  (d)->
    key: d["From\\To"]
    values: ({key: k, value: +d[k]} for k in Object.keys(d).filter((d)-> d isnt 'From\\To'))
  (data)->
    drawStateTransition('body', data)
