'use strict'
# 그리기 영역 정보 
margin = new Margin(50)
[width, height] = [600, 100]

hourInterval = 3
#------------------------------------------------------------
#
# ## showVideoList
#
# 동영상 메타데이터 시청회수 상위를 표시하는 핸들러 
#
# * @param  d 오브젝트 이벤트의 대상이 된 요소에 연결된 데이터 
#
showVideoList = (d)->
  # 리스트 요소를 작성
  $list = $('<ul>')
  # 자릿수 구분 포맷을 작성
  format = d3.format(',')
  # 시간 포맷 
  tFormat = d3.time.format('%Y/%m/%d(%a) %H')
  # 동영상 데이터 시청회수로 내림차순으로 정렬해서 상위 20건 
  for video in d.values.videos.sort((a, b)-> b.view_counter - a.view_counter).slice(0, 20)
    # 리스트 각 요소를 작성 
    $list.append $('<li>').append(
      $('<a>', {href: "http://www.nicovideo.jp/watch/#{video.video_id}"}).text(video.title).css('font-weight': 'bold')
      $('<span>').text(" (#{format(video.view_counter)} view / #{tFormat(new Date(video.upload_time))})")
    )
  # 이전 리스트 삭제 
  $('#videos .list ul').remove()
  # 새 리스트 추가 
  $('#videos .list').append $list

#------------------------------------------------------------
#
# ## showCommentList
#
# 댓글 대이터를 표시하는 핸들러 
#
# * @param  d 오브젝트 이벤트 대상이 된 요소에 연결하는 데이터
#
showCommentList = (d)->
  # 리스트 요소를 작성 
  $list = $('<ul>')
  # 댓글 데이터 선두 20건 
  for comment in d.values.comments.sort((a, b)-> a.no - b.no).slice(0, 20)
    # 리스트의 각 요소를 작성
    $list.append $('<li>').append(
      $('<span>').text(comment.comment).css('font-weight': 'bold')
    )
  # 이전 리스트 삭제 
  $('#comments .list ul').remove()
  # 새 리스트 추가 
  $('#comments .list').append $list

#------------------------------------------------------------
#
# ## createCalenderView
#
# 캘린더 뷰를 작성하는 함수 
#
# * @param  target           문자열  캘린더뷰를 추가하는 요소의 XPath
# * @param  data             배열    데이터 배열 
# * @param  mouseoverHandler 함수    마우스 오버 이벤트에 대한 핸들러
#
createCalenderView = (target, data, mouseoverHandler)->
  # 데이터의 범위를 가져온다 
  xExtent = d3.extent data, (d)-> +d.key
  yExtent = [0, 24 / hourInterval - 1]
  # 값의 최댓값을 가져온다 
  valueMax = d3.max data, (day)-> d3.max day.values, (hour)-> hour.values.v

  # 칸의 크기를 계산
  cellSize = width / (d3.range(xExtent[0], xExtent[1], 1000 * 60 * 60 * 24).length + 1)

  # x、y 및 색의 스케일을 작성 
  x = d3.time.scale().domain(xExtent).range([0, width])
  y = d3.scale.linear().domain(yExtent).range([height, 0])
  color = d3.scale.linear().domain([0, valueMax / 2, valueMax]).range(["#0000FF", "#FFFFFF", "#FF0000"])

  # svg 요소를 작성 
  svg = d3.select("#{target} .visualization").append("svg")
    .attr("width", width + margin.width).attr("height", height + margin.height)
    .append("g").attr("width", width).attr("height", height)
    .attr("transform", "translate(#{margin.left},#{margin.top})")

  # 날짜에 대응하는 그룹 요소를 작성 
  day = svg.selectAll('g').data(data).enter()
    .append('g')
    .attr('transform', (d)-> "translate(#{x(+d.key)}, 0)")

  # 시간별 셀 작성
  hour = day.selectAll('rect').data((d)-> d.values).enter()
    .append('rect').attr('width', cellSize).attr('height', cellSize)
    .attr('y', (d)-> cellSize * +d.key)
    .attr('fill', (d)-> color(d.values.v)).attr('stroke', 'grey')
    .style('cursor', 'pointer')

  # x축, y축의 값을 작성 
  xaxis = d3.svg.axis().scale(x).tickFormat(d3.time.format("%m/%d"))
  xAxis = svg.append('g').call(xaxis).attr('class', 'axis')
    .attr('transform', "translate(0, #{8 * cellSize})")
  yText = svg.selectAll('text.yaxis').data([yExtent[0].. yExtent[1]]).enter()
    .append('text').classed('yaxis', true).text((d)-> "#{d * 3} - #{(d + 1) * 3}")
    .attr('dy', (d, idx)-> cellSize * (idx + 1)).attr('dx', -50)

  # 핸들러가 있을 때는 핸들러로 지정 
  # 클릭으로 토글 
  if mouseoverHandler
    m = mouseoverHandler
    hour.on('mouseover.showlist', m)
    .on('click', (d)->
      m = if m then null else mouseoverHandler
      hour.on('mouseover.showlist', m)
    )


#------------------------------------------------------------
#
# # 비디오 데이터를 가져와서 캘린더 뷰의 히트맵 작성 
#
$.get "video.dat", (text)->
  videos = []
  for line in text.split(/\r?\n/)
    try
      videos.push JSON.parse(line)
    catch err
      err

  videos = videos.filter (d)-> new Date(d.upload_time) < new Date(2007, 4, 0)

  dataByDayAndHour = d3.nest()
    .key((d)-> d3.time.day(new Date(d.upload_time)).getTime())
    .sortKeys((a, b)-> a - b)
    .key((d)-> 0|new Date(d.upload_time).getHours() / hourInterval) # 정수화 (0|소수)
    .sortKeys((a, b)-> a - b)
    .rollup((values)->
      v: values.length
      videos: values
    )
    .entries(videos)

  createCalenderView("#videos", dataByDayAndHour, showVideoList)

#
# # 댓글 데이터를 가져와서 캘린더 뷰의 히트맵 작성
#
$.get "comment.dat", (text)->
  comments = []
  for line in text.split(/\r?\n/)
    try
      obj = JSON.parse(line)
      comments.push obj if new Date(obj.date * 1000) < new Date(2007, 4, 0)
    catch err
      err

  dataByDayAndHour = d3.nest()
    .key((d)-> d3.time.day(new Date(d.date * 1000)).getTime())
    .sortKeys((a, b)-> a - b)
    .key((d)-> 0|new Date(d.date * 1000).getHours() / hourInterval) # 정수화 (0|소수)
    .sortKeys((a, b)-> a - b)
    .rollup((values)->
      v: values.length
      comments: values
    )
    .entries(comments)

  createCalenderView("#comments", dataByDayAndHour, showCommentList)
